function ll = nbinll(data,X,omega,theta)
 
%NBINLL: Log-likelihood of model with negative binomial error structure.
%
%   Syntax:  LL = NBINDEV(DATA,X,OMEGA,THETA)
%
%   Input: DATA is an array containing the raw count data. X is the model
%   expectation. OMEGA and THETA are overdispersion parameters describing
%   the extra-Poisson variation as: OMEGA*X + THETA*X^2. The inputs OMEGA
%   and THETA must be scalars or matrices of the same size as DATA and X.
%
%   Output: LL is the log-likelihood of the parameters given data.
%
%   NB! Requires function nbinpdf.m in MATLAB Statistics Toolbox.
%
%   By Andreas Linden (andreas.linden@iki.fi)
 
% Transform input to column vectors
data = data(:);
X = X(:);
omega = omega(:);
theta = theta(:);
 
% Check syntax and input format
if nargin < 4
    error('All four input arguments must be provided.')
elseif isempty(data) || ~isa(data,'double')
    error('DATA must be a numerical array of class double.')
elseif any(data < 0) || any(round(data)-data)
    error('DATA should be counts, containing integers only.')
elseif isempty(X) || ~isa(X,'double')
    error('X must be a numerical array of class double.')
elseif any(X < 0)
    error('The model expectations X can not be negative.')
elseif numel(data) ~= numel(X)
    error('DATA and X must match in size.')
elseif ~isa(omega,'double') || ~isa(theta,'double')
    error('OMEGA and THETA must be a numerical arrays of class double.')
elseif ~any(numel(omega) == [1 numel(X)])
    error('OMEGA must be a scalar or match in size with DATA and X.')
elseif ~any(numel(theta) == [1 numel(X)])
    error('THETA must be a scalar or match in size with DATA and X.')
end
 
% Check that the combination of OMEGA and THETA is feasible
if any(omega+theta.*X <= 1)
    error('The given combination of OMEGA and THETA is not defined.');
end
 
% Calulcate parameters r and p of the negative binomial
r = X./(omega-1+theta.*X);
p = 1./(omega+theta.*X);
 
% Calculate log-likelihood
prob = nbinpdf(data,r,p);
ll = sum(log(prob));
 
end % File nbinll.m end
